
#include "Drv_DcModule.h"

#define DcModuleMaxVolt   (7500)//750V
#define DcModuleMinVolt   (2500)//250V
#define DcModuleMaxCurrt  (200) //20A
#define DcModuleMinCurrt  (40)  //4A

typedef struct
{
  u8  Id;
  u8  Temp;
  u16 Volt;
  u16 Currt;
  u16 State;
  u32 RecvTime;
}StructDcModuleStatus;


const StructDcModuleAbility DcModuleAbility =
{
  .MaxVolt = DcModuleMaxVolt,
  .MinVolt = DcModuleMinVolt,
  .MaxCurrt = DcModuleMaxCurrt,
  .MinCurrt = DcModuleMinCurrt,
};

StructDcModuleManage DcModuleManage;
StructDcModuleStatus DcModuleStatus[64];


void DcModuleRxIrqHandle(void)
{
  StructDcModuleStatus Data;
  UnionLittleEnd32 Temp;

  Data.Id = CANRID(DcModule_Can) & 0x000000FF;

  Temp.Mult32 = CANRDA(DcModule_Can);
  Data.Volt = (Temp.Disp32.LL32<<8) + Temp.Disp32.ML32;
  Data.Currt = (Temp.Disp32.MH32<<8) + Temp.Disp32.HH32;

  Temp.Mult32 = CANRDB(DcModule_Can);
  Data.Temp= (Temp.Disp32.LL32<<8) + Temp.Disp32.ML32;
  Data.State = (Temp.Disp32.MH32<<8) + Temp.Disp32.HH32;
  Data.RecvTime = GetSystemTick();

  if(Data.Id < 64)
  {
    DcModuleStatus[Data.Id] = Data;
    DcModuleManage.OnlineFlag |= ((long long)1<<Data.Id);
  }
}

void DcModuleCanInit(void)
{
  StructPinParam PinParam;

  PinParam.Dir = ePinDir_Out;
  PinParam.Mode = ePinMode_PullNone;
  PinParam.Func = DcModuleTx_PinFunc;
  PinParam.Pin = DcModuleTx_Pin;
  PinInit(DcModuleTx_Port, &PinParam);

  PinParam.Dir = ePinDir_In;
  PinParam.Mode = ePinMode_PullNone;
  PinParam.Func = DcModuleRx_PinFunc;
  PinParam.Pin = DcModuleRx_Pin;
  PinInit(DcModuleRx_Port, &PinParam);

  SetCanRxIrq(DcModule_Can, DcModuleRxIrqHandle);

  PeriphPowerSet(DcModule_CanPower, ENABLE);

  PeriphClkSet(DcModule_CanClk, ePclkDiv_CclkDiv_2);

  CANMOD(DcModule_Can) = 0x01;//CANģʽĴ,ЩֻCANMOD.0 == 1ʱſд

  CANIER(DcModule_Can) = 0x00;//жʹܼĴֹж
  CANGSR(DcModule_Can) = 0x00;// /* Clear status register */

  CANCMR(DcModule_Can) = 0x00;/* Reset CAN2 CMD register*/
  CANBTR(DcModule_Can) = 0x001C0008;//ʣ250K-- BitRate = Fpclk/( (BRP+1) * (1+(Tseg1+1)+(Tseg2+1)) )
  CANIER(DcModule_Can) = 0x01;//CAN1жʹܼĴʹܽжԼж
  CANEWL(DcModule_Can) = 0x7F;//޼Ĵ

  CANMOD(DcModule_Can) = 0x00;//շģʽ

  CAN_AFMR = 2;/* By default filter is not used */
}

/**************************************************************************************************
** : CAN1_SendMessage
** 䡡: ulCanId----ID ulCanDataA----ݵĵ4ֽڣ  ulCanDataB----ݵĸ4ֽ
** 䡡: ȡ״̬
**   : ͨCANͨ
** ա: 2010/08/19
**************************************************************************************************/
u8 DcModuleSendMsg(StructDcModuleSend *pMsg)
{
	u32 CANStatus;
  u32 CanSendFormat;

  CanSendFormat = ((((uint32)pMsg->Len) << 16) & 0x000F0000) | 0x80000000; //(bit31=1Ϊչ֡)  (bit30 RTR = 0Ϊ֡)
                               //29bitID,Զ֡3λݣȼΪ1
  CANStatus = CANSR(DcModule_Can);

	if(CANStatus & 0x00000004) 	//1
	{
		CANTFI(DcModule_Can, 0) = CanSendFormat;		//1 0 0000000000 0011 00000000 00000001

		CANTID(DcModule_Can, 0) = pMsg->Id;
		CANTDA(DcModule_Can, 0) = (pMsg->Data[0]) | (pMsg->Data[1]<<8) | (pMsg->Data[2]<<16) | (pMsg->Data[3]<<24);
		CANTDB(DcModule_Can, 0) = (pMsg->Data[4]) | (pMsg->Data[5]<<8) | (pMsg->Data[6]<<16) | (pMsg->Data[7]<<24);
		CANCMR(DcModule_Can) = 0x21;				//Tx1	   21
		return TRUE;
	}
	else if(CANStatus & 0x00000400)//2
	{
		CANTFI(DcModule_Can, 1) = CanSendFormat;

		CANTID(DcModule_Can, 1) = pMsg->Id;
		CANTDA(DcModule_Can, 1) = (pMsg->Data[0]) | (pMsg->Data[1]<<8) | (pMsg->Data[2]<<16) | (pMsg->Data[3]<<24);
		CANTDB(DcModule_Can, 1) = (pMsg->Data[4]) | (pMsg->Data[5]<<8) | (pMsg->Data[6]<<16) | (pMsg->Data[7]<<24);

		CANCMR(DcModule_Can) = 0x41;				//Tx2	   41
		return TRUE;
	}
	else if(CANStatus & 0x00040000)	//3
	{
		CANTFI(DcModule_Can, 2) = CanSendFormat;		//1 0 0000000000 0011 00000000 00000001

		CANTID(DcModule_Can, 2) = pMsg->Id;
		CANTDA(DcModule_Can, 2) = (pMsg->Data[0]) | (pMsg->Data[1]<<8) | (pMsg->Data[2]<<16) | (pMsg->Data[3]<<24);
		CANTDB(DcModule_Can, 2) = (pMsg->Data[4]) | (pMsg->Data[5]<<8) | (pMsg->Data[6]<<16) | (pMsg->Data[7]<<24);

		CANCMR(DcModule_Can) = 0x81;				//Tx1	   21
		return TRUE;
	}
	return FALSE;				//δͳɹFALSE
}

StructDcModuleCtrl DcModuleCtrl;

void DcModuleManageProcess(void)
{
  static u32 CheckTick = 0;

  u8 i;
  long long Flag;
  u8 Count;
  u32 Currt;

  if((GetSystemTick() - CheckTick) < 750)
    return;

  CheckTick = GetSystemTick();

  Flag = 1;
  Count = 0;
  for(i=0; i<64; i++)
  {
    if(Flag & DcModuleManage.OnlineFlag)
    {
      if((GetSystemTick() - DcModuleStatus[i].RecvTime) > 2000)
      {
        DcModuleManage.OnlineFlag &= ~Flag;
        DcModuleManage.ValidFlag &= ~Flag;
      }
      else
      {
        if(0x0000 == (DcModuleStatus[i].State & 0x0FFF))
        {
          DcModuleManage.ValidFlag |= Flag;
          Count++;
        }
      }
    }
    Flag = Flag << 1;
  }
  DcModuleManage.ValidNum = Count;

  Flag = 1;
  Currt = 0;
  for(i=0; i<64; i++)
  {
    if(Flag & DcModuleManage.ValidFlag)
    {
      Currt += DcModuleStatus[i].Currt;
      DcModuleCtrl.ActualVolt = DcModuleStatus[i].Volt;
    }
    Flag = Flag << 1;
  }
  DcModuleCtrl.ActualCurrt = Currt;

  if(0 == DcModuleManage.OnlineFlag)
    DcModuleCanInit();
}

void DcModuleCtrlProcess(void)
{
  static u32 CtrlTick = 0;
  static u8 LastFlag = 0;

  StructDcModuleSend DataSend;
  u32 Temp;

  Temp = 0;
  if(LastFlag != DcModuleCtrl.CtrlFlag)
  {
    LastFlag = DcModuleCtrl.CtrlFlag;
    Temp++;
  }
  if((GetSystemTick() - CtrlTick) > 100)
  {
    CtrlTick = GetSystemTick();
    Temp++;
  }
  if(0 == Temp)
    return;

  if(DcModuleCtrl.CtrlFlag)
  {
    if(DcModuleCtrl.NeedVolt > DcModuleAbility.MaxVolt)
      DcModuleManage.SetVolt = DcModuleAbility.MaxVolt;
    else if(DcModuleCtrl.NeedVolt > DcModuleAbility.MinVolt)
      DcModuleManage.SetVolt = DcModuleCtrl.NeedVolt;
    else
      DcModuleManage.SetVolt = DcModuleAbility.MinVolt;

    if(DcModuleManage.ValidNum)
    {
      Temp = DcModuleCtrl.NeedCurrt/DcModuleManage.ValidNum;
      if(DcModuleCtrl.NeedCurrt % DcModuleManage.ValidNum)
        Temp++;
    }
    else
      Temp = DcModuleManage.SetCurrt;
    if(Temp > DcModuleAbility.MaxCurrt)
      Temp = DcModuleAbility.MaxCurrt;
    else if(Temp < DcModuleAbility.MinCurrt)
      Temp = DcModuleAbility.MinCurrt;

    if(Temp <= DcModuleManage.SetCurrt)
      DcModuleManage.SetCurrt = Temp;
    else
      DcModuleManage.SetCurrt++;
    if(DcModuleManage.SetCurrt > DcModuleAbility.MaxCurrt)
      DcModuleManage.SetCurrt = DcModuleAbility.MaxCurrt;
    else if(DcModuleManage.SetCurrt < DcModuleAbility.MinCurrt)
      DcModuleManage.SetCurrt = DcModuleAbility.MinCurrt;

    DcModuleManage.Cmd = 0xAA;//
  }
  else
  {
    DcModuleManage.Cmd = 0x00;//ػ
    DcModuleManage.SetVolt = 0;
    DcModuleManage.SetCurrt = 0;
  }

  DataSend.Id = 0x01013FAA;
  DataSend.Len = 5;
  DataSend.Data[0] = (DcModuleManage.SetVolt/100) >> 8;
  DataSend.Data[1] = (DcModuleManage.SetVolt/100);
  DataSend.Data[2] = (DcModuleManage.SetCurrt/100) >> 8;
  DataSend.Data[3] = (DcModuleManage.SetCurrt/100);
  DataSend.Data[4] = DcModuleManage.Cmd;
  DcModuleSendMsg(&DataSend);
}

/* the system main thread */
void DcModule(void const *parameter)
{
  osDelay(2000);

  while(1)
  {
    DcModuleManageProcess();
    osDelay(20);
    DcModuleCtrlProcess();
    osDelay(20);
  }
}

void DcModuleSet(EnumSwitchStatus Set, u16 Volt, u16 Currt)
{
  if(eSwSta_Off == Set)
  {
    DcModuleCtrl.CtrlFlag = 0;
    DcModuleCtrl.NeedVolt = 0;
    DcModuleCtrl.NeedCurrt = 0;
  }
  else
  {
    DcModuleCtrl.CtrlFlag = 1;
    DcModuleCtrl.NeedVolt = Volt;
    DcModuleCtrl.NeedCurrt = Currt;
  }
}

u16 GetDcModuleVolt(void)
{
  return DcModuleCtrl.ActualVolt;
}

u16 GetDcModuleCurrt(void)
{
  return DcModuleCtrl.ActualCurrt;
}

u16 GetDcModuleVoltSet(void)
{
  return DcModuleCtrl.NeedVolt;
}

u8 DcModuleInit(void)
{
  DcModuleCanInit();

  osThreadDef(DcModule, osPriorityAboveNormal, 0, configMINIMAL_STACK_SIZE);
  if(NULL == osThreadCreate(osThread(DcModule), NULL))
    return FALSE;

  return TRUE;
}



