
#include "Drv_MCP2515.h"

 //SPI
#define Cmd_RESET         0xC0
#define Cmd_READ          0x03
#define Cmd_READ_RX       0x90
#define Cmd_WRITE         0x02
#define Cmd_WRITE_TX      0x40
#define Cmd_RTS           0x80
#define Cmd_READ_STATUS   0xA0
#define Cmd_RX_STATUS     0xB0
#define Cmd_BIT_MODIFY    0x05


#define MCP2515Sta_RX0IF    (0x01)
#define MCP2515Sta_RX1IF    (0x02)
#define MCP2515Sta_TX0REQ   (0x04)
#define MCP2515Sta_TX0IF    (0x08)
#define MCP2515Sta_TX1REQ   (0x10)
#define MCP2515Sta_TX1IF    (0x20)
#define MCP2515Sta_TX2REQ   (0x40)
#define MCP2515Sta_TX2IF    (0x80)


pMCP2515IntHandle IntHandle = NULL;

pMCP2515IntHandle IntHandle_B = NULL;

/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Initializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void MCP2515_SpiLowLevelInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Enable the SPI clock */
  MCP2515_SPI_CLK_INIT(MCP2515_SPI_CLK, ENABLE);

  /*!< Enable GPIO clocks */
  RCC_APB2PeriphClockCmd(MCP2515_SPI_SCK_GPIO_CLK | MCP2515_SPI_MISO_GPIO_CLK |
                         MCP2515_SPI_MOSI_GPIO_CLK | MCP2515_CS_GPIO_CLK, ENABLE);

  /*!< SPI pins configuration *************************************************/
  /*!< Connect SPI pins to AF5 */
  GPIO_PinAFConfig(MCP2515_SPI_SCK_GPIO_PORT, MCP2515_SPI_SCK_SOURCE, MCP2515_SPI_SCK_AF);
  GPIO_PinAFConfig(MCP2515_SPI_MISO_GPIO_PORT, MCP2515_SPI_MISO_SOURCE, MCP2515_SPI_MISO_AF);
  GPIO_PinAFConfig(MCP2515_SPI_MOSI_GPIO_PORT, MCP2515_SPI_MOSI_SOURCE, MCP2515_SPI_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;

  /*!< SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = MCP2515_SPI_SCK_PIN;
  GPIO_Init(MCP2515_SPI_SCK_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  MCP2515_SPI_MOSI_PIN;
  GPIO_Init(MCP2515_SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MISO pin configuration */
  GPIO_InitStructure.GPIO_Pin =  MCP2515_SPI_MISO_PIN;
  GPIO_Init(MCP2515_SPI_MISO_GPIO_PORT, &GPIO_InitStructure);

  /*!< Configure sFLASH Card CS pin in output pushpull mode ********************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = MCP2515_CS_PIN;
  GPIO_Init(MCP2515_CS_GPIO_PORT, &GPIO_InitStructure);
}

/* Exported macro ------------------------------------------------------------*/
/* Select sFLASH: Chip Select pin low */
void MCP2515_SpiSelect(void)
{
  GPIO_ResetBits(MCP2515_CS_GPIO_PORT, MCP2515_CS_PIN);
}

/* Deselect sFLASH: Chip Select pin high */
void MCP2515_SpiDeselect(void)
{
  GPIO_SetBits(MCP2515_CS_GPIO_PORT, MCP2515_CS_PIN);
}

/**
  * @brief  Initializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
u8 MCP2515_SpiInit(void)
{
  SPI_InitTypeDef  SPI_InitStructure;

  MCP2515_SpiLowLevelInit();

  /*!< Deselect the FLASH: Chip Select high */
  MCP2515_SpiDeselect();

  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(MCP2515_SPI, &SPI_InitStructure);

  /*!< Enable the MCP2515_SPI  */
  SPI_Cmd(MCP2515_SPI, ENABLE);

  return TRUE;
}
u8 MCP2515_SpiHightSpeedInit(void)
{
  SPI_InitTypeDef  SPI_InitStructure;

  MCP2515_SpiLowLevelInit();

  /*!< Deselect the FLASH: Chip Select high */
  MCP2515_SpiDeselect();

  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(MCP2515_SPI, &SPI_InitStructure);

  /*!< Enable the MCP2515_SPI  */
  SPI_Cmd(MCP2515_SPI, ENABLE);

  return TRUE;
}

/**
  * @brief  Sends a byte through the SPI interface and return the byte received
  *         from the SPI bus.
  * @param  byte: byte to send.
  * @retval The value of the received byte.
  */
u8 MCP2515_SpiSendByte(u8 byte)
{
  /*!< Loop while DR register in not emplty */
  while (SPI_I2S_GetFlagStatus(MCP2515_SPI, SPI_I2S_FLAG_TXE) == RESET);

  /*!< Send byte through the SPI1 peripheral */
  SPI_I2S_SendData(MCP2515_SPI, byte);

  /*!< Wait to receive a byte */
  while (SPI_I2S_GetFlagStatus(MCP2515_SPI, SPI_I2S_FLAG_RXNE) == RESET);

  /*!< Return the byte read from the SPI bus */
  return SPI_I2S_ReceiveData(MCP2515_SPI);
}

void MCP2515_Reset(void)
{
  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_RESET);

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable
}

//**********************************************************//
//   ˵MCP2515ƼĴ                      //
//   룺    Ĵַ                             //
//       Ĵ                                  //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
u8 MCP2515_ReadReg(u8 Addr)
{
  u8 Data;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_READ); // SPIдĴ
  MCP2515_SpiSendByte(Addr); //ͼĴַ
  Data = MCP2515_SpiSendByte(0xFF); //ضĴ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable

  return Data;
}

//**********************************************************//
//   ˵MCP2515ջ                      //
//   룺    ַ                             //
//                                         //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
u8 MCP2515_ReadRxBuffer(u8 NumRecvbuf, u8 *pData, u8 Len)
{
  u8 i;

  // жadressǷЧ12λ඼ӦΪ0
  if((NumRecvbuf > 1) || (NULL == pData) || (0 == Len))
    return FALSE;
  if(Len > 13)
    Len = 13;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  if(0 == NumRecvbuf)
    MCP2515_SpiSendByte(Cmd_READ_RX);
  else
    MCP2515_SpiSendByte(Cmd_READ_RX | 0x04); //Ͷȡ
  for(i=0; i<Len; i++)
    *pData++ = MCP2515_SpiSendByte(0xFF);  //ݱ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable

  return TRUE;
}

//**********************************************************//
//   ˵MCP2515дƼĴ                      //
//   룺    Ĵַд                         //
//                                                 //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
void MCP2515_WriteReg(u8 Addr, u8 Data)
{
  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_WRITE); // SPIдĴ
  MCP2515_SpiSendByte(Addr);  //ͼĴַ
  MCP2515_SpiSendByte(Data);   //ͼĴ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable
}

//**********************************************************//
//   ˵MCP2515Ĵд            //
//   룺    Ĵʼַָ룬ݳ      //
//                                             //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
u8 MCP2515_WriteTxBuffer(u8 NumSendBuf, u8 *pData, u8 Len)
{
  u8 i;

  if((NumSendBuf > 2) || (NULL == pData) || (0 == Len))
    return FALSE;
  if(Len > 13)
    Len = 13;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  if(0 == NumSendBuf)
    MCP2515_SpiSendByte(Cmd_WRITE_TX);  //SPIдָ
  else if(1 == NumSendBuf)
    MCP2515_SpiSendByte(Cmd_WRITE_TX | 0x02);  //SPIдָ
  else
     MCP2515_SpiSendByte(Cmd_WRITE_TX| 0x04);  //SPIдָ

  for (i=0; i<Len ;i++ )
    MCP2515_SpiSendByte(*pData++);  //

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable

  return TRUE;
}

//**********************************************************//
//   ˵MCP2515ջ                      //
//   룺    ַ                             //
//                                         //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
u8 MCP2515_ReqSend(u8 Num)
{
  if (Num > 2)// жadressǷЧ12λ඼ӦΪ0
    return FALSE;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_RTS | (0x01 << Num)); //Ͷȡ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable

  return TRUE;
}

//**********************************************************//
//   ˵MCP2515ƼĴ                      //
//   룺    Ĵַ                             //
//       Ĵ                                  //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
u8 MCP2515_ReadStatus(void)
{
  u8 Data;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_READ_STATUS); // SPIдĴ
  Data = MCP2515_SpiSendByte(0xFF); //ضĴ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable

  return Data;
}

u8 MCP2515_ReadRxStatus(void)
{
  u8 Data;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_RX_STATUS); // SPIдĴ
  Data = MCP2515_SpiSendByte(0xFF); //ضĴ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable

  return Data;
}

//**********************************************************//
//   ˵MCP2515ƼĴλ޸ĳ                  //
//   룺    Ĵַ޸λ޸                 //
//                                                  //
//   úSPIͳMCP2515_SendByte                  //
//**********************************************************//
void MCP2515_BitModify(u8 Addr, u8 Mask, u8 Data)
{
  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_BIT_MODIFY); //SPIλ޸ָ
  MCP2515_SpiSendByte(Addr);    //ͼĴַ
  MCP2515_SpiSendByte(Mask);     //ֽڣ//ֽ"1"ʾӦλ޸ģ"0"ʾֹ޸
  MCP2515_SpiSendByte(Data);     //ֽ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable
}

//**********************************************************//
//   ˵MCP2515Ĵ            //
//   룺    Ĵʼַָ룬ݳ      //
//                                             //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
void MCP2515_ReadBuffer(u8 Addr, u8 *pData, u8 Len)
{
  u8 i;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_READ);  //SPIָ
  MCP2515_SpiSendByte(Addr);   //ʼĴַ
  for(i=0; i<Len; i++)
    *pData++ = MCP2515_SpiSendByte(0xFF);  //ݱ

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable
}

//**********************************************************//
//   ˵MCP2515Ĵд            //
//   룺    Ĵʼַָ룬ݳ      //
//                                             //
//   úSPIͳMCP2515_SendByte                //
//**********************************************************//
void MCP2515_WriteBuffer(u8 Addr, u8 *pData, u8 Len)
{
  u8 i;

  MCP2515_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_SpiSendByte(Cmd_WRITE);  //SPIдָ
  MCP2515_SpiSendByte(Addr);    //ʼĴַ
  for (i=0; i<Len ;i++ )
    MCP2515_SpiSendByte(*pData++);  //

  MCP2515_SpiDeselect();//CS high ,MCP2515 disable
}



void MCP2515_INT_EXTI_IRQHandle(void)
{
  if(EXTI_GetITStatus(MCP2515_INT_EXTI_Line) != RESET)
  {
    while(MCP2515_INT_PIN != (MCP2515_INT_GPIO_PORT->IDR & MCP2515_INT_PIN))
    {
      if(NULL != IntHandle)
        IntHandle();
    }

    EXTI_ClearITPendingBit(MCP2515_INT_EXTI_Line);/* Clear the EXTI line 0 pending bit */
  }
}

void MCP2515_EnterCritical(void)
{
  NVIC_InitTypeDef   NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = MCP2515_INT_EXTI_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;

  NVIC_Init(&NVIC_InitStructure);
}

void MCP2515_ExitCritical(void)
{
  NVIC_InitTypeDef   NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = MCP2515_INT_EXTI_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);
}


void MCP2515_SetInt(pMCP2515IntHandle pIrqHandle, FunctionalState NewState)
{
  EXTI_InitTypeDef   EXTI_InitStructure;
  GPIO_InitTypeDef   GPIO_InitStructure;
  NVIC_InitTypeDef   NVIC_InitStructure;

  /* Enable GPIOA clock */
  RCC_AHB1PeriphClockCmd(MCP2515_INT_GPIO_CLK, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  /* Enable SYSCFG clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  /* Configure PA0 pin as input floating */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Pin = MCP2515_INT_PIN;
  GPIO_Init(MCP2515_INT_GPIO_PORT, &GPIO_InitStructure);

  /* Connect EXTI Line0 to PA0 pin */
  SYSCFG_EXTILineConfig(MCP2515_INT_EXTI_PortSource, MCP2515_INT_EXTI_PinSource);

  /* Configure EXTI Line0 */
  EXTI_InitStructure.EXTI_Line = MCP2515_INT_EXTI_Line;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;

  /* Enable and set EXTI Line0 Interrupt to the lowest priority */
  NVIC_InitStructure.NVIC_IRQChannel = MCP2515_INT_EXTI_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;

  if(DISABLE == NewState)
  {
    EXTI_InitStructure.EXTI_LineCmd = DISABLE;
    NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
  }
  else
  {
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  }

  EXTI_Init(&EXTI_InitStructure);
  NVIC_Init(&NVIC_InitStructure);

  IntHandle = pIrqHandle;
}



u8 MCP2515_Transmit(CanTxMsg* TxMessage)
{
  u8 transmit_mailbox = 0;
  u8 Status = 0;
  u8 Data[14];

  MCP2515_EnterCritical();

  Status = MCP2515_ReadStatus();

  /* Select one empty transmit mailbox */
  if(MCP2515Sta_TX0REQ != (Status & MCP2515Sta_TX0REQ))
    transmit_mailbox = 0;
  else if(MCP2515Sta_TX1REQ != (Status & MCP2515Sta_TX1REQ))
    transmit_mailbox = 1;
  else if(MCP2515Sta_TX2REQ != (Status & MCP2515Sta_TX2REQ))
    transmit_mailbox = 2;
  else
    transmit_mailbox = CAN_TxStatus_NoMailBox;

  if(transmit_mailbox != CAN_TxStatus_NoMailBox)
  {
    if(CAN_ID_EXT == TxMessage->IDE)
    {
      Data[0] = (u8)(TxMessage->ExtId >> 21);
      Data[1] = (u8)(((TxMessage->ExtId>>13)&0xE0) | 0x08 | ((TxMessage->ExtId>>16)&0x03));
      Data[2] = (u8)(TxMessage->ExtId >> 8);
      Data[3] = (u8)(TxMessage->ExtId >> 0);
    }
    else
    {
      Data[0] = (u8)(TxMessage->StdId >> 3);
      Data[1] = (u8)((TxMessage->StdId<<5) & 0xE0);
      Data[2] = 0;
      Data[3] = 0;
    }

    if(CAN_RTR_REMOTE == TxMessage->RTR)
      Data[4] = (u8)((TxMessage->DLC & 0x0F) | 0x40);
    else
      Data[4] = (u8)(TxMessage->DLC & 0x0F);

    Data[5] = TxMessage->Data[0];
    Data[6] = TxMessage->Data[1];
    Data[7] = TxMessage->Data[2];
    Data[8] = TxMessage->Data[3];
    Data[9] = TxMessage->Data[4];
    Data[10] = TxMessage->Data[5];
    Data[11] = TxMessage->Data[6];
    Data[12] = TxMessage->Data[7];

    MCP2515_WriteTxBuffer(transmit_mailbox, Data, 13);
    MCP2515_ReqSend(transmit_mailbox);
  }

  MCP2515_ExitCritical();

  return transmit_mailbox;
}

u8 MCP2515_Receive(u8 FIFONumber, CanRxMsg* RxMessage)
{
  u8 Data[14] = {0};
  u32 Temp;

  MCP2515_EnterCritical();

  if(FALSE == MCP2515_ReadRxBuffer(FIFONumber, Data, 13))
  {
    MCP2515_ExitCritical();
    return FALSE;
  }

  if(Data[1] & 0x08)
    RxMessage->IDE = CAN_ID_EXT;
  else
    RxMessage->IDE = CAN_ID_STD;

  if(Data[4] & 0x40)
    RxMessage->RTR = CAN_RTR_REMOTE;
  else
    RxMessage->RTR = CAN_RTR_DATA;

  RxMessage->DLC = Data[4] & 0x0F;

  if (CAN_ID_EXT == RxMessage->IDE)
  {
    Temp = 0;
    Temp |= (u32)((u32)Data[0] << 21);
    Temp |= (u32)((u32)(Data[1]&0xE0) << 13);
    Temp |= (u32)((u32)(Data[1]&0x03) << 16);
    Temp |= (u32)((u32)(Data[2]) << 8);
    Temp |= (u32)((u32)(Data[3]) << 0);
    RxMessage->ExtId = Temp;
  }
  else
  {
    Temp = 0;
    Temp |= (u32)((u32)Data[0] << 3);
    Temp |= (u32)((u32)(Data[1]&0xE0) >> 5);
    RxMessage->StdId = Temp;
  }

  /* Get the data field */
  RxMessage->Data[0] = Data[5];
  RxMessage->Data[1] = Data[6];
  RxMessage->Data[2] = Data[7];
  RxMessage->Data[3] = Data[8];
  RxMessage->Data[4] = Data[9];
  RxMessage->Data[5] = Data[10];
  RxMessage->Data[6] = Data[11];
  RxMessage->Data[7] = Data[12];

  MCP2515_ExitCritical();

  return TRUE;
}

//**********************************************************//
//                                                          //
//                                                          //
//                                                          //
//                                                          //
//**********************************************************//

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Initializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void MCP2515_B_SpiLowLevelInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Enable the SPI clock */
  MCP2515_B_SPI_CLK_INIT(MCP2515_B_SPI_CLK, ENABLE);

  /*!< Enable GPIO clocks */
  RCC_APB2PeriphClockCmd(MCP2515_B_SPI_SCK_GPIO_CLK | MCP2515_B_SPI_MISO_GPIO_CLK |
                         MCP2515_B_SPI_MOSI_GPIO_CLK | MCP2515_B_CS_GPIO_CLK, ENABLE);

  /*!< SPI pins configuration *************************************************/
  /*!< Connect SPI pins to AF5 */
  GPIO_PinAFConfig(MCP2515_B_SPI_SCK_GPIO_PORT, MCP2515_B_SPI_SCK_SOURCE, MCP2515_B_SPI_SCK_AF);
  GPIO_PinAFConfig(MCP2515_B_SPI_MISO_GPIO_PORT, MCP2515_B_SPI_MISO_SOURCE, MCP2515_B_SPI_MISO_AF);
  GPIO_PinAFConfig(MCP2515_B_SPI_MOSI_GPIO_PORT, MCP2515_B_SPI_MOSI_SOURCE, MCP2515_B_SPI_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;

  /*!< SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = MCP2515_B_SPI_SCK_PIN;
  GPIO_Init(MCP2515_B_SPI_SCK_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  MCP2515_B_SPI_MOSI_PIN;
  GPIO_Init(MCP2515_B_SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MISO pin configuration */
  GPIO_InitStructure.GPIO_Pin =  MCP2515_B_SPI_MISO_PIN;
  GPIO_Init(MCP2515_B_SPI_MISO_GPIO_PORT, &GPIO_InitStructure);

  /*!< Configure sFLASH Card CS pin in output pushpull mode ********************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = MCP2515_B_CS_PIN;
  GPIO_Init(MCP2515_B_CS_GPIO_PORT, &GPIO_InitStructure);
}
/* Exported macro ------------------------------------------------------------*/
/* Select sFLASH: Chip Select pin low */
void MCP2515_B_SpiSelect(void)
{
  GPIO_ResetBits(MCP2515_B_CS_GPIO_PORT, MCP2515_B_CS_PIN);
}

/* Deselect sFLASH: Chip Select pin high */
void MCP2515_B_SpiDeselect(void)
{
  GPIO_SetBits(MCP2515_B_CS_GPIO_PORT, MCP2515_B_CS_PIN);
}

/**
  * @brief  Initializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
u8 MCP2515_B_SpiInit(void)
{
  SPI_InitTypeDef  SPI_InitStructure;

  MCP2515_B_SpiLowLevelInit();

  /*!< Deselect the FLASH: Chip Select high */
  MCP2515_B_SpiDeselect();

  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(MCP2515_B_SPI, &SPI_InitStructure);

  /*!< Enable the MCP2515_SPI  */
  SPI_Cmd(MCP2515_B_SPI, ENABLE);

  return TRUE;
}
u8 MCP2515_B_SpiHightSpeedInit(void)
{
  SPI_InitTypeDef  SPI_InitStructure;

  MCP2515_B_SpiLowLevelInit();

  /*!< Deselect the FLASH: Chip Select high */
  MCP2515_B_SpiDeselect();

  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(MCP2515_B_SPI, &SPI_InitStructure);

  /*!< Enable the MCP2515_SPI  */
  SPI_Cmd(MCP2515_B_SPI, ENABLE);

  return TRUE;
}

/**
  * @brief  Sends a byte through the SPI interface and return the byte received
  *         from the SPI bus.
  * @param  byte: byte to send.
  * @retval The value of the received byte.
  */
u8 MCP2515_B_SpiSendByte(u8 byte)
{
  /*!< Loop while DR register in not emplty */
  while (SPI_I2S_GetFlagStatus(MCP2515_B_SPI, SPI_I2S_FLAG_TXE) == RESET);

  /*!< Send byte through the SPI1 peripheral */
  SPI_I2S_SendData(MCP2515_B_SPI, byte);

  /*!< Wait to receive a byte */
  while (SPI_I2S_GetFlagStatus(MCP2515_B_SPI, SPI_I2S_FLAG_RXNE) == RESET);

  /*!< Return the byte read from the SPI bus */
  return SPI_I2S_ReceiveData(MCP2515_B_SPI);
}

void MCP2515_B_Reset(void)
{
  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_RESET);

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable
}

u8 MCP2515_B_ReadReg(u8 Addr)
{
  u8 Data;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_READ); // SPIдĴ
  MCP2515_B_SpiSendByte(Addr); //ͼĴַ
  Data = MCP2515_B_SpiSendByte(0xFF); //ضĴ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable

  return Data;
}

u8 MCP2515_B_ReadRxBuffer(u8 NumRecvbuf, u8 *pData, u8 Len)
{
  u8 i;

  // жadressǷЧ12λ඼ӦΪ0
  if((NumRecvbuf > 1) || (NULL == pData) || (0 == Len))
    return FALSE;
  if(Len > 13)
    Len = 13;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  if(0 == NumRecvbuf)
    MCP2515_B_SpiSendByte(Cmd_READ_RX);
  else
    MCP2515_B_SpiSendByte(Cmd_READ_RX | 0x04); //Ͷȡ
  for(i=0; i<Len; i++)
    *pData++ = MCP2515_B_SpiSendByte(0xFF);  //ݱ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable

  return TRUE;
}

void MCP2515_B_WriteReg(u8 Addr, u8 Data)
{
  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_WRITE); // SPIдĴ
  MCP2515_B_SpiSendByte(Addr);  //ͼĴַ
  MCP2515_B_SpiSendByte(Data);   //ͼĴ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable
}

u8 MCP2515_B_WriteTxBuffer(u8 NumSendBuf, u8 *pData, u8 Len)
{
  u8 i;

  if((NumSendBuf > 2) || (NULL == pData) || (0 == Len))
    return FALSE;
  if(Len > 13)
    Len = 13;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  if(0 == NumSendBuf)
    MCP2515_B_SpiSendByte(Cmd_WRITE_TX);  //SPIдָ
  else if(1 == NumSendBuf)
    MCP2515_B_SpiSendByte(Cmd_WRITE_TX | 0x02);  //SPIдָ
  else
     MCP2515_B_SpiSendByte(Cmd_WRITE_TX| 0x04);  //SPIдָ

  for (i=0; i<Len ;i++ )
    MCP2515_B_SpiSendByte(*pData++);  //

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable

  return TRUE;
}

u8 MCP2515_B_ReqSend(u8 Num)
{
  if (Num > 2)// жadressǷЧ12λ඼ӦΪ0
    return FALSE;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_RTS | (0x01 << Num)); //Ͷȡ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable

  return TRUE;
}

u8 MCP2515_B_ReadStatus(void)
{
  u8 Data;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_READ_STATUS); // SPIдĴ
  Data = MCP2515_B_SpiSendByte(0xFF); //ضĴ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable

  return Data;
}

u8 MCP2515_B_ReadRxStatus(void)
{
  u8 Data;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_RX_STATUS); // SPIдĴ
  Data = MCP2515_B_SpiSendByte(0xFF); //ضĴ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable

  return Data;
}

void MCP2515_B_WriteBuffer(u8 Addr, u8 *pData, u8 Len)
{
  u8 i;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_WRITE);  //SPIдָ
  MCP2515_B_SpiSendByte(Addr);    //ʼĴַ
  for (i=0; i<Len ;i++ )
    MCP2515_B_SpiSendByte(*pData++);  //

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable
}

void MCP2515_B_ReadBuffer(u8 Addr, u8 *pData, u8 Len)
{
  u8 i;

  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_READ);  //SPIָ
  MCP2515_B_SpiSendByte(Addr);   //ʼĴַ
  for(i=0; i<Len; i++)
    *pData++ = MCP2515_B_SpiSendByte(0xFF);  //ݱ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable
}

void MCP2515_B_EnterCritical(void)
{
  NVIC_InitTypeDef   NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = MCP2515_B_INT_EXTI_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;

  NVIC_Init(&NVIC_InitStructure);
}

void MCP2515_B_BitModify(u8 Addr, u8 Mask, u8 Data)
{
  MCP2515_B_SpiSelect();// CS low ,MCP2515 enable

  MCP2515_B_SpiSendByte(Cmd_BIT_MODIFY); //SPIλ޸ָ
  MCP2515_B_SpiSendByte(Addr);    //ͼĴַ
  MCP2515_B_SpiSendByte(Mask);     //ֽڣ//ֽ"1"ʾӦλ޸ģ"0"ʾֹ޸
  MCP2515_B_SpiSendByte(Data);     //ֽ

  MCP2515_B_SpiDeselect();//CS high ,MCP2515 disable
}

void MCP2515_B_ExitCritical(void)
{
  NVIC_InitTypeDef   NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = MCP2515_B_INT_EXTI_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);
}

void MCP2515_B_SetInt(pMCP2515IntHandle pIrqHandle, FunctionalState NewState)
{
  EXTI_InitTypeDef   EXTI_InitStructure;
  GPIO_InitTypeDef   GPIO_InitStructure;
  NVIC_InitTypeDef   NVIC_InitStructure;

  /* Enable GPIOB clock */
  RCC_AHB1PeriphClockCmd(MCP2515_B_INT_GPIO_CLK, ENABLE);
  /* Enable SYSCFG clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  /* Configure PB0 pin as input floating */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Pin = MCP2515_B_INT_PIN;
  GPIO_Init(MCP2515_B_INT_GPIO_PORT, &GPIO_InitStructure);

  /* Connect EXTI Line0 to PB0 pin */
  SYSCFG_EXTILineConfig(MCP2515_B_INT_EXTI_PortSource, MCP2515_B_INT_EXTI_PinSource);

  /* Configure EXTI Line0 */
  EXTI_InitStructure.EXTI_Line = MCP2515_B_INT_EXTI_Line;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;

  /* Enable and set EXTI Line0 Interrupt to the lowest priority */
  NVIC_InitStructure.NVIC_IRQChannel = MCP2515_B_INT_EXTI_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;

  if(DISABLE == NewState)
  {
    EXTI_InitStructure.EXTI_LineCmd = DISABLE;
    NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
  }
  else
  {
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  }

  EXTI_Init(&EXTI_InitStructure);
  NVIC_Init(&NVIC_InitStructure);

  IntHandle_B = pIrqHandle;
}

u8 MCP2515_B_Transmit(CanTxMsg* TxMessage)
{
  u8 transmit_mailbox = 0;
  u8 Status = 0;
  u8 Data[14];

  MCP2515_B_EnterCritical();

  Status = MCP2515_B_ReadStatus();

  /* Select one empty transmit mailbox */
  if(MCP2515Sta_TX0REQ != (Status & MCP2515Sta_TX0REQ))
    transmit_mailbox = 0;
  else if(MCP2515Sta_TX1REQ != (Status & MCP2515Sta_TX1REQ))
    transmit_mailbox = 1;
  else if(MCP2515Sta_TX2REQ != (Status & MCP2515Sta_TX2REQ))
    transmit_mailbox = 2;
  else
    transmit_mailbox = CAN_TxStatus_NoMailBox;

  if(transmit_mailbox != CAN_TxStatus_NoMailBox)
  {
    if(CAN_ID_EXT == TxMessage->IDE)
    {
      Data[0] = (u8)(TxMessage->ExtId >> 21);
      Data[1] = (u8)(((TxMessage->ExtId>>13)&0xE0) | 0x08 | ((TxMessage->ExtId>>16)&0x03));
      Data[2] = (u8)(TxMessage->ExtId >> 8);
      Data[3] = (u8)(TxMessage->ExtId >> 0);
    }
    else
    {
      Data[0] = (u8)(TxMessage->StdId >> 3);
      Data[1] = (u8)((TxMessage->StdId<<5) & 0xE0);
      Data[2] = 0;
      Data[3] = 0;
    }

    if(CAN_RTR_REMOTE == TxMessage->RTR)
      Data[4] = (u8)((TxMessage->DLC & 0x0F) | 0x40);
    else
      Data[4] = (u8)(TxMessage->DLC & 0x0F);

    Data[5] = TxMessage->Data[0];
    Data[6] = TxMessage->Data[1];
    Data[7] = TxMessage->Data[2];
    Data[8] = TxMessage->Data[3];
    Data[9] = TxMessage->Data[4];
    Data[10] = TxMessage->Data[5];
    Data[11] = TxMessage->Data[6];
    Data[12] = TxMessage->Data[7];

    MCP2515_B_WriteTxBuffer(transmit_mailbox, Data, 13);
    MCP2515_B_ReqSend(transmit_mailbox);
  }

  MCP2515_B_ExitCritical();

  return transmit_mailbox;
}

u8 MCP2515_B_Receive(u8 FIFONumber, CanRxMsg* RxMessage)
{
  u8 Data[14] = {0};
  u32 Temp;

  MCP2515_B_EnterCritical();

  if(FALSE == MCP2515_B_ReadRxBuffer(FIFONumber, Data, 13))
  {
    MCP2515_B_ExitCritical();
    return FALSE;
  }

  if(Data[1] & 0x08)
    RxMessage->IDE = CAN_ID_EXT;
  else
    RxMessage->IDE = CAN_ID_STD;

  if(Data[4] & 0x40)
    RxMessage->RTR = CAN_RTR_REMOTE;
  else
    RxMessage->RTR = CAN_RTR_DATA;

  RxMessage->DLC = Data[4] & 0x0F;

  if (CAN_ID_EXT == RxMessage->IDE)
  {
    Temp = 0;
    Temp |= (u32)((u32)Data[0] << 21);
    Temp |= (u32)((u32)(Data[1]&0xE0) << 13);
    Temp |= (u32)((u32)(Data[1]&0x03) << 16);
    Temp |= (u32)((u32)(Data[2]) << 8);
    Temp |= (u32)((u32)(Data[3]) << 0);
    RxMessage->ExtId = Temp;
  }
  else
  {
    Temp = 0;
    Temp |= (u32)((u32)Data[0] << 3);
    Temp |= (u32)((u32)(Data[1]&0xE0) >> 5);
    RxMessage->StdId = Temp;
  }

  /* Get the data field */
  RxMessage->Data[0] = Data[5];
  RxMessage->Data[1] = Data[6];
  RxMessage->Data[2] = Data[7];
  RxMessage->Data[3] = Data[8];
  RxMessage->Data[4] = Data[9];
  RxMessage->Data[5] = Data[10];
  RxMessage->Data[6] = Data[11];
  RxMessage->Data[7] = Data[12];

  MCP2515_B_ExitCritical();

  return TRUE;
}


void MCP2515_B_INT_EXTI_IRQHandle(void)
{
  if(EXTI_GetITStatus(MCP2515_B_INT_EXTI_Line) != RESET)
  {
    while(MCP2515_B_INT_PIN != (MCP2515_B_INT_GPIO_PORT->IDR & MCP2515_B_INT_PIN))
    {
      if(NULL != IntHandle_B)
        IntHandle_B();
    }

    EXTI_ClearITPendingBit(MCP2515_B_INT_EXTI_Line);/* Clear the EXTI line 0 pending bit */
  }
}


