
#include "Thd_Charge_B.h"

#include "Drv_BoardIo.h"
#include "Drv_DcModule.h"
#include "Drv_BmsComm.h"
#include "Drv_Meter.h"

#include "Drv_ImdJyc.h"

#define ImdCheckFreq          (200)//ms
#define DcModOutTimeOut       (11000)//ms
#define VoltWaitTimeOut       (1000)//ms
#define ChkWaitTimeOut        (12000)//(8500)//ms
#define ChkTimeOut            (1500)//ms
#define ImdCheckCount         (3)
#define BleederCheckTimeOut   (3500)//ms

typedef enum
{
  eImdStep_Start= 0,
  eImdStep_DcModOut,
  eImdStep_VoltWait,
  eImdStep_ChkWait,
  eImdStep_Check,
  eImdStep_Bleeder,
  eImdStep_Stop,
  eImdStep_Finish,
}EnumImdChkStep;

/**************************************************************************************************
** : GetImdCheck
** 䡡: 
** 䡡: 0 δм    1 ͨ    -1 Ե;   -2 Ե
**   : ȡԵ
**************************************************************************************************/
s32 GetImdCheck_B(void)
{
  u32 PositResist;
  u32 NegatResist;
  u32 Volt;
  u32 Resist;

  if(0 == ImdValue_B.Vaild)
    return 0;

  PositResist = ImdValue_B.PositResist;
  NegatResist = ImdValue_B.NegatResist;
  Volt = GetDcModuleVolt_B();

  if(PositResist < NegatResist)
    Resist = (PositResist*10000);
  else
    Resist = (NegatResist*10000);

  if(Resist >= (505*Volt))
    return 1;
  else if(Resist >= (108*Volt))
    return -1;
  else
    return -2;
}

/**************************************************************************************************
** : GetBleederResult
** 䡡: 
** 䡡: 0 δмͨ    1 ͨ
**   : ȡйŵ·
**************************************************************************************************/
u8 GetBleederResult_B(void)
{
  if(GetDcModuleVolt_B() < 600)
    return TRUE;
  return FALSE;
}

void ImdCheckSet_B(EnumSwitchStatus Set)
{
//  if(eSwSta_Off == Set)
//    ImdWrite_B(0x0017, 0x0003);
//  else
//    ImdWrite_B(0x0017, 0x0002);
//  ImdWrite_B(0x01FE, 0x0001);
}

void ImdProcess_B(EnumImdChkStep *pImdChckStep)
{
  static u32 ImdCheckTick = 0;
  static u32 ImdTimeout = 0;
  static u8 ImdNoErrCnt = 0;
  static u8 ImdWarnCnt = 0;
  static u8 ImdErrCnt = 0;

  s32 Tmep;

  if((GetSystemTick() - ImdCheckTick) < ImdCheckFreq)
    return;

  ImdCheckTick = GetSystemTick();

  switch(*pImdChckStep)
  {
    case eImdStep_Start:
      DcOutSet_B(eSwSta_On);
      ImdWorkSet_B(eSwSta_On);
      ImdCheckSet_B(eSwSta_Off);
      //ImdCheckSet(eSwSta_On);
      if(0 != BhmMsg_B.Valid)//ѹЧ
      {
        if(BhmMsg_B.ImdMaxVolt <= DcModuleAbility.MaxVolt)
          Tmep = BhmMsg_B.ImdMaxVolt;
        else
          Tmep = DcModuleAbility.MaxVolt;
      }
      else
        Tmep = DcModuleAbility.MinVolt;

      DcModuleSet_B(eSwSta_On, Tmep, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      *pImdChckStep = eImdStep_DcModOut;
      ImdTimeout = 0;
      break;
    case eImdStep_DcModOut:
      if(0 == ImdTimeout)
        ImdTimeout = GetSystemTick();
      else if((GetSystemTick() - ImdTimeout) > DcModOutTimeOut)
      {
        *pImdChckStep = eImdStep_VoltWait;
        ImdTimeout = 0;
      }
      if(GetDcModuleVolt_B() >= ((GetDcModuleVoltSet_B()>>3)*7.8))
      {
        *pImdChckStep = eImdStep_VoltWait;
        ImdTimeout = 0;
      }
      break;
    case eImdStep_VoltWait:
      if(0 == ImdTimeout)
        ImdTimeout = GetSystemTick();
      else if((GetSystemTick() - ImdTimeout) > VoltWaitTimeOut)
      {
        ImdCheckSet_B(eSwSta_On);
        *pImdChckStep = eImdStep_ChkWait;
        ImdTimeout = 0;
      }
      if(ImdValue_B.PnVolt >= ((GetDcModuleVolt_B()>>3)*7.8))
      {
        ImdCheckSet_B(eSwSta_On);
        *pImdChckStep = eImdStep_ChkWait;
        ImdTimeout = 0;
      }
      break;
    case eImdStep_ChkWait:
      if(0 == ImdTimeout)
        ImdTimeout = GetSystemTick();
      else if((GetSystemTick() - ImdTimeout) > ChkWaitTimeOut)
      {
        *pImdChckStep = eImdStep_Check;
        ImdTimeout = 0;
        ImdNoErrCnt = 0;
        ImdWarnCnt = 0;
        ImdErrCnt = 0;
      }
      break;
    case eImdStep_Check:
      if(0 == ImdTimeout)
        ImdTimeout = GetSystemTick();
      else if((0 != ImdTimeout) && ((GetSystemTick() - ImdTimeout) > ChkTimeOut))
      {//ʱ
        ChargeError_B.Value.IsolateWarn = 1;//Ե
        ChargeError_B.Value.IsolateError = 0;

        DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);//ֱģ
        osDelay(100);
        DrainResistSet_B(eSwSta_On);
        ImdCheckSet_B(eSwSta_Off);
        *pImdChckStep = eImdStep_Bleeder;
        ImdTimeout = 0;
      }
      Tmep = GetImdCheck_B();
      if(0 != Tmep)
      {
        if(1 == Tmep)
          ImdNoErrCnt++;
        else if(-1 == Tmep)
          ImdWarnCnt++;
        else
          ImdErrCnt++;

        if(ImdNoErrCnt >= ImdCheckCount)
        {
          ChargeError_B.Value.IsolateWarn = 0;//Ե
          ChargeError_B.Value.IsolateError = 0;//Ե
          DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);//ֱģ
          osDelay(100);
          DrainResistSet_B(eSwSta_On);
          ImdCheckSet_B(eSwSta_Off);
          *pImdChckStep = eImdStep_Bleeder;
          ImdTimeout = 0;
        }
        else if(ImdWarnCnt >= ImdCheckCount)
        {
          ChargeError_B.Value.IsolateWarn = 1;//Ե
          ChargeError_B.Value.IsolateError = 0;//Ե
          DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);//ֱģ
          osDelay(100);
          DrainResistSet_B(eSwSta_On);
          ImdCheckSet_B(eSwSta_Off);
          *pImdChckStep = eImdStep_Bleeder;
          ImdTimeout = 0;
        }
        else if(ImdErrCnt >= ImdCheckCount)
        {
          ChargeError_B.Value.IsolateWarn = 0;//Ե
          ChargeError_B.Value.IsolateError = 1;//Ե
          DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);//ֱģ
          osDelay(100);
          DrainResistSet_B(eSwSta_On);
          ImdCheckSet_B(eSwSta_Off);
          *pImdChckStep = eImdStep_Bleeder;
          ImdTimeout = 0;
        }
      }
      break;
    case eImdStep_Bleeder:
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);//ֱģ
      DcOutSet_B(eSwSta_Off);
      DrainResistSet_B(eSwSta_On);
      ImdWorkSet_B(eSwSta_Off);
      ImdCheckSet_B(eSwSta_Off);

      if(0 == ImdTimeout)
        ImdTimeout = GetSystemTick();
      if((GetSystemTick() - ImdTimeout) > BleederCheckTimeOut)
      {//ʱ
        ChargeError_B.Value.DrainResist = 1;//йŵ辯
        DrainResistSet_B(eSwSta_Off);
        *pImdChckStep = eImdStep_Stop;
      }

      if(FALSE != GetBleederResult_B())
      {
        ChargeError_B.Value.DrainResist = 0;//йŵ辯
        DrainResistSet_B(eSwSta_Off);
        *pImdChckStep = eImdStep_Stop;
      }
      break;
    case eImdStep_Stop:
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);//ֱģ
      DcOutSet_B(eSwSta_Off);
      ImdWorkSet_B(eSwSta_Off);
      ImdCheckSet_B(eSwSta_Off);
      DrainResistSet_B(eSwSta_Off);
      *pImdChckStep = eImdStep_Finish;
      break;
    case eImdStep_Finish:
      break;
    default:
      *pImdChckStep = eImdStep_Start;
      break;
  }
}

void ChgProImdCheck_B(StructChargeCtrl_B *pChargeCtrl)
{
  StructPgnRecv PgnRecv;
  u32 CHM_SendTick = 0;
  u32 CcStatusChkTick = 0;

  EnumImdChkStep ImdChckStep;

  PillarError_B.Value.Insulation = 0;
  PillarError_B.Value.ImdDev = 0;
  PillarError_B.Value.DrainResist = 0;

  ChargeError_B.Value.DrainResist = 0;
  ChargeError_B.Value.IsolateWarn = 0;//Ե
  ChargeError_B.Value.IsolateError = 0;
  ChargeError_B.Value.VoltOutError = 0;
  ChargeError_B.Value.CurrtOutError = 0;

  BmsCanInit_B();
  osDelay(20);

  ChargeTimerSet_B(eSwSta_Off);
  CleanChgTimeCnt_B();
  //BmsPowerSet(eSwSta_On);
  LockSet_B(eSwSta_On);
  FanSet_B(eSwSta_On);
  ImdWorkSet_B(eSwSta_On);
  DrainResistSet_B(eSwSta_Off);
  DcOutSet_B(eSwSta_Off);

  CHM_Send_B();
  CHM_SendTick = GetSystemTick();
  BhmMsg_B.RecvTick = GetSystemTick();

  ImdChckStep = eImdStep_Start;

  while(1)
  {
    if((GetSystemTick() - CHM_SendTick) > 250)
    {
      if(eChgVer_2011 != ChgVer_B)
        CHM_Send_B();
      CHM_SendTick = GetSystemTick();
    }

    ChgPgnRecv_B(&PgnRecv);

    ImdProcess_B(&ImdChckStep);

    if(eImdStep_Finish == ImdChckStep)
    {
      if(1 == ChargeError_B.Value.IsolateError)
      {
        ChargeStopType_B = eChgStop_Imd_IsolateError;
        pChargeCtrl->CurProcess = eChgPro_B_ChgStop;
        pChargeCtrl->Param = 0;
      }
      else
      {
        pChargeCtrl->CurProcess = eChgPro_B_ChgRecognize;
        pChargeCtrl->Param = 0;
      }
    }

    if((eChgVer_2011 != ChgVer_B) && ((GetSystemTick() - BhmMsg_B.RecvTick) > MsgTimeout_B_5S))
    {//ͨŹ
      ChargeStopType_B = eChgStop_Imd_BhmTimeOut;
      pChargeCtrl->CurProcess = eChgPro_B_ChgStop;
      pChargeCtrl->Param = 0;
    }

    if(0 == TCUCurChgRecd_B.TCUChargeEnable)
    {//ⲿƽ
      ChargeStopType_B = eChgStop_Imd_UserCancel;
      pChargeCtrl->CurProcess = eChgPro_B_ChgStop;
      pChargeCtrl->Param = 0;
    }

    if(eSwSta_On != CcStatusRead_B())
    {
      if(0 == CcStatusChkTick)
        CcStatusChkTick = GetSystemTick();
    }
    else
      CcStatusChkTick = 0;
    if((0 != CcStatusChkTick) && ((GetSystemTick() - CcStatusChkTick) > CC_CheckTick_B * 2))
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      CcStatusChkTick = 0;
      ChargeStopType_B = eChgStop_Imd_CcOpen;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.Emergency)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_EmerClose;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.Door)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_DoorOpen;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.Contactor)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_Contactor;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.UniContactor)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_Contactor;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.InVolt)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_InVolt;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.NoDcmod)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_NoDcmod;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(0 != PillarError_B.Value.Meter)
    {
      DcModuleSet_B(eSwSta_Off, DcModuleAbility.MinVolt, DcModuleAbility.MinCurrt*DcModuleManage_B.ValidNum);
      DcOutSet_B(eSwSta_Off);
      ChargeStopType_B = eChgStop_Imd_Meter;
      pChargeCtrl->CurProcess = eChgPro_B_ChgBreak;
      pChargeCtrl->Param = 0;
    }
    if(eChgPro_B_ImdCheck != pChargeCtrl->CurProcess)
      return;

    osDelay(25);
  }
}


