
#include "Thd_Charge.h"

#include "Drv_BoardIo.h"
#include "Drv_DcModule.h"
#include "Drv_BmsComm.h"
#include "Memory.h"
#include "Drv_Meter.h"
#include "Drv_Adc.h"
#include "Drv_CommuSPI2.h"

#include "Drv_Dht12.h"




u8 ChargeEnable = 0;
u8 ChargePauseFlag = 0;
u8 UniChgEnable = FALSE;
s32 StartChgEnergy = 0;
s32 StopChgEnergy = 0;

u8 ChargeEnable_AB= 0;

EnumChgStopType ChargeStopType = eChgStop_None;

u8 CommErrorCount = 0;
u32 CommErrFrame[4] = {0};

extern void ChgProChgIdle(StructChargeCtrl *Param);
extern void ChgProShakeHand(StructChargeCtrl *Param);

extern void ChgProImdCheck(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgRecognizeEx(StructChargeCtrl *pChargeCtrl);

extern void ChgProChgRecognize(StructChargeCtrl *Param);
extern void ChgProChgConfigure(StructChargeCtrl *pChargeCtrl);
extern void ChgProWaitBmsReady(StructChargeCtrl *pChargeCtrl);
extern void ChgProWaitPillarReady(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgStart(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgRun(StructChargeCtrl *pChargeCtrl);
extern void ChgProWaitVoltChgRun(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgStop(StructChargeCtrl *pChargeCtrl);
extern void ChgProCommBeak(StructChargeCtrl *pChargeCtrl);
extern void ChgProCommError(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgBreak(StructChargeCtrl *pChargeCtrl);

extern void ChgProShakeHand_AB(StructChargeCtrl *Param);

extern void ChgProImdCheck_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgRecognizeEx_AB(StructChargeCtrl *pChargeCtrl);

extern void ChgProChgRecognize_AB(StructChargeCtrl *Param);
extern void ChgProChgConfigure_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProWaitBmsReady_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProWaitPillarReady_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgStart_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgRun_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProWaitVoltChgRun_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgStop_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProCommBeak_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProCommError_AB(StructChargeCtrl *pChargeCtrl);
extern void ChgProChgBreak_AB(StructChargeCtrl *pChargeCtrl);

const pChargeCtrlHandle ChargeCtrlHandle[] =
{
  ChgProChgIdle,
  ChgProShakeHand,

  ChgProImdCheck,
  ChgProChgRecognizeEx,

  ChgProChgRecognize,
  ChgProChgConfigure,
  ChgProWaitBmsReady,
  ChgProWaitPillarReady,
  ChgProChgStart,
  ChgProChgRun,
  ChgProWaitVoltChgRun,
  ChgProChgStop,
  ChgProCommBeak,
  ChgProCommError,
  ChgProChgBreak,

  ChgProShakeHand_AB,

  ChgProImdCheck_AB,
  ChgProChgRecognizeEx_AB,

  ChgProChgRecognize_AB,
  ChgProChgConfigure_AB,
  ChgProWaitBmsReady_AB,
  ChgProWaitPillarReady_AB,
  ChgProChgStart_AB,
  ChgProChgRun_AB,
  ChgProWaitVoltChgRun_AB,
  ChgProChgStop_AB,
  ChgProCommBeak_AB,
  ChgProCommError_AB,
  ChgProChgBreak_AB,

  ChgProChgIdle
};

StructChargeCtrl ChargeCtrl;

osTimerId ChargeTimer;
u32 ChargeTimeCount;//
u8 ChargeTimeCountEn = 0;

void ChargeTimerSet(EnumSwitchStatus OnOff)
{
  if(eSwSta_Off == OnOff)
    ChargeTimeCountEn = 0;
  else
    ChargeTimeCountEn = 1;
}

u16 GetChgMin(void)
{
  u32 Temp;

  Temp = ChargeTimeCount/60;
  if(Temp > 600)
    Temp = 600;
  return Temp;
}

u32 GetChgSec(void)
{
  return ChargeTimeCount;
}


void CleanChgTimeCnt(void)
{
  ChargeTimeCount = 0;
}

void ChargeTimeCountCtrl(void const *Param)
{
  if(ChargeTimeCountEn)
    ChargeTimeCount++;
}

/* the system main thread */
void Thd_Charge(void const *parameter)
{
  CommuSPI2_LowLevel_Init();
  MemoryInit();

  DcModuleInit();//DcModuleInit_B();
  //MeterInit();
  //DcOutInit_Uni();

  DcOutInit();   //DcOutInit_B();
  FanInit();
  FanInit_Top();
  DehumidifierInit();
  //DrainResistInit();//DrainResistInit_B();
  ACContactorInit(); //交流接触器
  LockInit();       //LockInit_B();
  EmergenceInit();
  DoorChkInit();
  SPDInit();
  IdleLedInit();//  IdleLedInit_B();
  ChgLedInit();//ChgLedInit_B();
  FaultLedInit();//FaultLedInit_B();
  RunLedInit();
  AdcCheckInit();

  BmsPowerInit();  //BmsPowerInit_B();
  BmsCommInit();   //BmsCommInit_B();
  ShortCirCheck();


  /* Create a Timer */
  osTimerDef(ChargeTimer, ChargeTimeCountCtrl);
  ChargeTimer = osTimerCreate(osTimer(ChargeTimer), osTimerPeriodic, (void *)0);
  osTimerStart(ChargeTimer, 1000);//һ
  ChargeCtrl.CurProcess = eChgPro_ChgIdle;
  ChargeCtrl.Param = 0;

  osDelay(100);

  while(1)
  {
    if(ChargeCtrl.CurProcess < eChgPro_Max)
      ChargeCtrlHandle[ChargeCtrl.CurProcess](&ChargeCtrl);
    else
    {
      ChargeCtrl.CurProcess = eChgPro_ChgIdle;
      ChargeCtrl.Param = 0;
    }

    osDelay(5);
  }
}

